/* -*-	Mode:C++; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*- */
/*
 * Copyright (c) 1994 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the Computer Systems
 *	Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @(#) $Header: /nfs/jade/vint/CVSROOT/ns-2/queue/drop-tail.h,v 1.17 2002/01/01 04:26:10 sfloyd Exp $ (LBL)
 */
// HCF v3.0

#ifndef ns_drop_tail_h
#define ns_drop_tail_h

#include <string.h>
#include "queue.h"
#include "config.h"

/*
 * A bounded, drop-tail queue
 */
class DropTail : public Queue {
  public:
	DropTail() {
		q_ = new PacketQueue;
		pq_ = q_;
		bind_bool("drop_front_", &drop_front_);
		bind_bool("summarystats_", &summarystats);
		bind_bool("queue_in_bytes_", &qib_);  // boolean: q in bytes?
		bind("mean_pktsize_", &mean_pktsize_);
		//		_RENAMED("drop-front_", "drop_front_");
	}
	~DropTail() {
		delete q_;
	}
  protected:
	void reset();
	int command(int argc, const char*const* argv);
	void enque(Packet*);
	Packet* deque();
	PacketQueue *q_;	/* underlying FIFO queue */
	int drop_front_;	/* drop-from-front (rather than from tail) */
	int summarystats;
	void print_summarystats();
	int qib_;       	/* bool: queue measured in bytes? */
	int mean_pktsize_;	/* configured mean packet size in bytes */

};

//------------------------------------------------------------------------------------------------------------------------------//EDCFpatch

class BS_DropTail;

#include "channel.h"
#include "mac-802_11.h"

struct list_queue {             // The structure associated to each queue

	PacketQueue* que_;      // The queue object
   int tx_burst;           // When bursting mode is used, the number of frame to send in the current burst
   double timeout;
   int limit;
};

#define MAX_QUEUE 8             // Compatible with 8 Access Category in 802.11e


class BS_QueueHandler : public Handler {        // This class rapresents the Handler between Mac802_11 and BS_DropTail
        friend class BS_DropTail;
public:
   inline BS_QueueHandler(BS_DropTail& q) : queue_(q) {}
   void handle(Event*);
   void stop_bursting(int n);
   // ---------- HCFstart
   int queueLength(int ac);
   int queueByteLength(int ac);
   // ---------- HCFend

private:
	BS_DropTail& queue_;    // The corresponding BS_DropTail object
};


class BS_DropTail : public Queue {      // This class groups all queues of each node
        friend class BS_QueueHandler;
public:
        BS_DropTail(): num_queue(0), qtodq(0), qtoen(0), qbs_(*this) {
                bind_bool("queue_in_byte_", &qinb_);    // boolean: q in bytes?
                bind("mapping_APvoip", &mapping_APvoip);
                bind("mapping_APcbr", &mapping_APcbr);
                bind("mapping_APvbr", &mapping_APvbr);
                bind("mapping_APtcp", &mapping_APtcp);
                bind("mapping_APack", &mapping_APack);
                bind("mapping_WSvoip", &mapping_WSvoip);
                bind("mapping_WScbr", &mapping_WScbr);
                bind("mapping_WSvbr", &mapping_WSvbr);
                bind("mapping_WStcp", &mapping_WStcp);
                bind("mapping_WSack", &mapping_WSack);                           
        }

        ~BS_DropTail() {
                for (int i=0 ; i<num_queue ; i++) {
                        delete tab[i].que_;
                }
        }
        void recv(Packet* p, Handler*);
        void resume();

protected:
        int command(int argc, const char*const* argv);
        void dump_stat();
        // ------ Larcheri Paolo - April 2005 --------
        void dump_avg_size();
        void reset_avg_size();
        // -------------------------------------------
        void enque(Packet*);
        Packet* deque();
        Packet* force_deque(int n);
        PacketQueue* select(Packet*);
        int calc_bursting();

        int qtodq;                      // The identifier of the queue that is currently served
        int bursting;                   // A flag for selecting the bursting modality (0,-1 or #n#)
        list_queue tab[MAX_QUEUE];      // The list of the queues associated to this node
        int num_queue;                  // The number of queues (one for each Access Category in 802.11e)
        BS_QueueHandler qbs_;           // The handler between Mac802_11 and BS_DropTail
        bool block[MAX_QUEUE];          // A flag indicating whether the corresponding queue is empty (0) or not (1)
        int qtoen;                      // The identifier of the queue at which the incoming frame is destined
        int qinb_;

        int mapping_APvoip;
        int mapping_APcbr;
        int mapping_APvbr;
        int mapping_APtcp;
        int mapping_APack;        
        int mapping_WSvoip;
        int mapping_WScbr;
        int mapping_WSvbr;
        int mapping_WStcp;
        int mapping_WSack;
};

//----------------------------------------------------------------------------------------------------------------------------//EndEDCFpatch

#endif
