getopt $argc $argv opt       ;# Reads the command line parameters

set opt(seed)           777                         ;# Seed of the Random Number Generator
set opt(x)              [expr $opt(dia).0 * 1.1]    ;# X-size of the topography
set opt(y)              [expr $opt(dia).0 * 1.1]    ;# Y-size of the topography
set opt(bs)             1                           ;# Number of Base Station
set opt(err)            ChnErrProc                  ;# The Wireless Channel Error Model to use

if ![file exists $opt(dir)] {
        file mkdir $opt(dir)
}
set file(finish) [open "$opt(dir)/$opt(idx)_finish.txt" w]
                                                                    ;# Opening of output files
if {$opt(onoff) >0} {
        set file(web) [open "$opt(dir)/$opt(idx)_web.txt" w]
        puts $file(web) [format "#%7s %12s %12s %12s %12s %12s %12s %10s %9s %12s %12s %8s %8s %10s %10s\n" \
                flow_id direction pkt_to_send pkt_tx pkt_rcv ack_tx ack_rcv fast_rec timeout throughput goodput avg_wnd max_wnd delay variance]
}

source "errmodel.tcl"

#-----------------------------------------------------------------------#
#            Wireless Network Interface  parameters                     #
#-----------------------------------------------------------------------#

set opt(chan)   Channel/WirelessChannel
set opt(netif)  Phy/WirelessPhy

Phy/WirelessPhy set CPThresh_   10              ;# Capture threshold (db)
Phy/WirelessPhy set CSThresh_   1.559e-11       ;# Carrier sense threshold (W)
Phy/WirelessPhy set RXThresh_   3.652e-10       ;# Receive power threshold (W)
Phy/WirelessPhy set bandwidth_  $opt(bandwidth) ;# Bit rate
Phy/WirelessPhy set Pt_         0.28183815      ;# Transmitted signal power (W)
Phy/WirelessPhy set freq_       914e+6          ;# Frequency
Phy/WirelessPhy set L_          1.0             ;# System loss factor

#-----------------------------------------------------------------------#
#        Radio Propagation Model and Antenna parameters                 #
#-----------------------------------------------------------------------#

set opt(prop)   Propagation/TwoRayGround
set opt(ant)    Antenna/OmniAntenna

Propagation/Shadowing set pathlossExp_  2.0             ;# Path-loss exponent
Propagation/Shadowing set std_db_       4.0             ;# Shadowing deviation (dB)
Propagation/Shadowing set dist0_        1.0             ;# Close-in reference distance
Propagation/Shadowing set seed_         $opt(seed)      ;# Seed for random number generator

Antenna/OmniAntenna set X_      0       ;#\__
Antenna/OmniAntenna set Y_      0       ;# __> Position w.r.t. the node
Antenna/OmniAntenna set Z_      1.5     ;#/
Antenna/OmniAntenna set Gt_     1.0     ;# Gain of transmitter (db)
Antenna/OmniAntenna set Gr_     1.0     ;# Gain of receiver (db)

#-----------------------------------------------------------------------#
#                       MAC parameters                                  #
#-----------------------------------------------------------------------#

set opt(mac)            Mac/802_11
set opt(macHC)		Mac/802_11HC

Mac/802_11   set basicRate_     1e6
Mac/802_11   set dataRate_      $opt(bandwidth)
Mac/802_11HC set basicRate_     1e6
Mac/802_11HC set dataRate_      $opt(bandwidth)

set datarate_fast	11e6		;# fast data transmission rate (11Mbps)
set datarate_slow	 2e6		;# slow data transmission rate ( 2Mbps)

if {$opt(vocoder) == "g729"} {		;# vocoder g.729:
	set voicepkt	20		;# voice packet size (20 bytes)
	set framing	0.020		;# vocoder framing (20 ms)
}

if {$opt(vocoder) == "g711"} {		;# vocoder g.711:
	set voicepkt	240		;# voice packet size (240 bytes)
	set framing	0.030		;# vocoder framing (30 ms)
}

#-----------------------------------------------------------------------#
#                       LLC parameters                                  #
#-----------------------------------------------------------------------#

set opt(ll)	LL

LL set delay_           25us  	;# Line latency

#-----------------------------------------------------------------------#
#             Interface Queue parameters                                #
#-----------------------------------------------------------------------#

set opt(ifq)    Queue/DropTail/BS

Queue/DropTail set drop_front_          false           ;# Drop-from-front (rather than from tail)
Queue/DropTail set queue_in_bytes_      false           ;# Queue measured in bytes?
Queue/DropTail set mean_pktsize_        $opt(tcp_size)  ;# Configured mean packet size in bytes

Queue/DropTail/BS set queue_in_byte_    $opt(qinb)

#-----------------------------------------------------------------------#
#                       Routing Agent parameters                        #
#-----------------------------------------------------------------------#

set opt(adhocRouting)   DSDV

Agent/DSDV set perup_ [expr $opt(stop) * 2]     ;# Period between updates
