# Traffic Agents configuration

Agent set fid_          0       ;# For IPv6 flow id field
Agent set agent_addr_   -1      ;# Address of this agent
Agent set agent_port_   -1      ;# Port of this agent
Agent set dst_addr_     -1      ;# Destination address for pkt flow
Agent set dst_port_     -1      ;# Destination port for pkt flow
Agent set ttl_          32      ;# Default ttl for outgoing pkts

Agent/TCP set maxcwnd_                  0       ;# Max # cwnd can ever be --- 0 to ignore (default)
Agent/TCP set numdupacks_               3       ;# Dup ACKs before fast retransmit
Agent/TCP set window_                   100     ;# The upper bound on the advertised window (default 20)
Agent/TCP set windowInit_               1       ;# The initial size of the congestion window on slow-start (default 2)
Agent/TCP set windowInitOption_         1       ;# Congestion avoidance algorithm --- 1 for using wnd_init_ --- 2 for
                                                ;#      using large initial windows
Agent/TCP set syn_                      0       ;# 1 for modeling SYN/ACK exchange  (default 1)
Agent/TCP set windowOption_             1       ;# The algorithm to use for managing the congestion window
Agent/TCP set ecn_                      0       ;# Explicit Congestion Notification. This allows a Fast Retransmit after a
                                                ;#      quench() due to an ECN bit
Agent/TCP set packetSize_        $opt(tcp_size) ;# Fixed packet size (bytes)
Agent/TCP set tcpip_base_hdr_size_      40      ;# Size of base TCP/IP header
Agent/TCP set slow_start_restart_       true    ;# Boolean: re-init cwnd after connection goes idle.  On by default
Agent/TCP set tcpTick_                  0.01    ;# Clock granularity for measuring roundtrip times
Agent/TCP set maxrto_                   100000  ;# Max value of an RTO (sec.)
Agent/TCP set minrto_                   1       ;# Min value of an RTO
Agent/TCP set srtt_init_                0       ;# Initial value for computing t_srtt_
Agent/TCP set rttvar_init_              12      ;# Initial value for computing t_rttvar_
Agent/TCP set rtxcur_init_              3.0     ;# Initial value for t_rtxcur_
Agent/TCP set T_SRTT_BITS               3       ;# Exponent of weight for updating t_srtt_
Agent/TCP set T_RTTVAR_BITS             2       ;# Exponent of weight for updating t_rttvar_
Agent/TCP set rttvar_exp_               2       ;# Exponent of multiple for t_rtxcur_
Agent/TCP set useHeaders_               true    ;# Boolean: Add TCP/IP header sizes

Agent/TCPSink set sport_                        0       ;# Source port
Agent/TCPSink set dport_                        0       ;# Destination port
Agent/TCPSink set packetSize_                   40      ;# Fixed packet size (bytes)
Agent/TCPSink set RFC2581_immediate_ack_        true    ;# Used to generate ACKs immediately for RFC2581-compliant gap-filling

Agent/TCP/Newreno set newreno_changes_          0       ;# 0 for fixing unnecessary fast retransmits; 1 for additional code from Allman,
                                                        ;# to implement other algorithms from Hoe's paper, including sending a new packet
                                                        ;# for every two duplicate ACKs. The default is set to 0.
Agent/TCP/Newreno set newreno_changes1_         0       ;# Newreno_changes1_ set to 0 gives the Slow-but-Steady variant of NewReno
                                                        ;# from RFC 2582, with the retransmit timer reset after each partial new ack.
                                                        ;# Newreno_changes1_ set to 1 gives the Impatient variant of NewReno from RFC 2582,
                                                        ;# with the retransmit timer reset only for the first partial new ack.
                                                        ;# The default is set to 0
Agent/TCP/Newreno set partial_window_deflation_ 0       ;# 0 if set cwnd to ssthresh upon partial new ack (default). 1 if deflate
                                                        ;# (cwnd + dupwnd) by amount of data acked "Partial window deflation" is discussed
                                                        ;# in RFC 2582.
Agent/TCP/Newreno set exit_recovery_fix_        0       ;# 0 for setting cwnd to ssthresh upon leaving fast recovery (default). 1 for
                                                        ;# setting cwnd to min(ssthresh, amnt. of data in network) when leaving

Agent/UDP set packetSize_ $opt(udp_size)        ;# Max. UDP packet size (bytes)

Application/Traffic/CBR set packetSize_         $opt(udp_size)                  ;# Fixed packet size (bytes)
Application/Traffic/CBR set random_             1                               ;# Introduces random noise in the scheduled departure times

######################################################################
#                                                                    #
# HTTP_model_init: when the TCP on/off model is used, this function  #
#                  is responsable of the model initialization        #
#                                                                    #
######################################################################

proc HTTP_model_init {} {
	global web avg opt
                                       ;#(bytes)

	set avg(flow_length,0)             119          ;# This flow length distribution is the result
	set avg(flow_length,1)             179          ;# of a real traffic measurement executed on the backbone
	set avg(flow_length,2)             251          ;# network of the Politecnico of Torino
	set avg(flow_length,3)             334
	set avg(flow_length,4)             428
	set avg(flow_length,5)             529
	set avg(flow_length,6)             658
	set avg(flow_length,7)             948
	set avg(flow_length,8)            1650
	set avg(flow_length,9)            2861
        set avg(flow_length,10)           4706
        set avg(flow_length,11)           8015
        set avg(flow_length,12)          13681
        set avg(flow_length,13)          26641
        set avg(flow_length,14)         284454
        
        set total 0
        set web(npkt) 0
        foreach k [array names avg "flow_length,*"] {
                incr total [expr 1 + int($avg($k)/$opt(tcp_size))]
                incr web(npkt)
        }
        set web(avg_ptos) [expr $total.0 / $web(npkt).0]        ;# computes the average flow length of the model

}

if {$opt(tcp) == 1 && $opt(onoff) != 2} {
        puts $file(finish) "#\n#\tTCP Flows :\n#"

        if {$opt(onoff) > 0} {
                HTTP_model_init
        }

        set num_tcp 0

        foreach j [range 1 $opt(nn)] {
                set tcp($j) [new Agent/TCP/Newreno]     ;# Creates a new TCP NewReno agent
                set obj(tcp,$j) $tcp($j)
                $tcp($j) set fid_ $j                    ;# Sets the TCP flow identifier
                set sink($j) [new Agent/TCPSink]        ;# Creates a new TCP Sink agent
                set obj(tcp_sink,$j) $sink($j)
                $sink($j) set fid_ $j                   ;# Sets the TCP flow identifier

                if {[expr $j % 4] == 3 || [expr $j % 4] == 0} {
                        puts $file(finish) [format "#\t\t%2d)  0-->WN[expr $j - 1]  :  down-link starting at %f" \
                                                        $j [expr 0.000050 * $num_tcp]]
                        $ns_ attach-agent $W(0) $tcp($j)
                        $ns_ attach-agent $node_($num_tcp) $sink($j)    ;# Down-Link flows: the source TCP agent is attached to the wired node
                        set fl($j) "0-->WN[expr $j - 1]"
                } else {
                        puts $file(finish) [format "#\t\t%2d)  WN[expr $j - 1]-->0  :    up-link starting at %f" \
                                                                $j [expr 0.000050 * $num_tcp]]
                        $ns_ attach-agent $node_($num_tcp) $tcp($j)
                        $ns_ attach-agent $W(0) $sink($j)            ;# Up-Link flows: the source TCP agent is attached to the wireless station
                        set fl($j) "WN[expr $j - 1]-->0"
                }

                $ns_ connect $tcp($j) $sink($j)         ;# Creates the transport level connection between the source and the destination
                set ftp($j) [new Application/FTP]       ;# Creates a new FTP agent
                $ftp($j) attach-agent $tcp($j)          ;# Connects the FTP agent over the corresponding TCP one
                if {$opt(onoff) == 0} {
                        $ns_ at [expr $opt(start) + [expr 0.000050 * $num_tcp]] "$ftp($j) start"
                        $ns_ at $opt(stop) "$ftp($j) stop"
                                                                ;# TCP greedy: departure times of TCP sources are diversified among
                                                                ;#             stations to avoid a common collision at the beginning
                } else {
                        $ns_ at [expr $opt(start) + [expr 0.000050 * $num_tcp]] "start_HTTP_flows $j"
                }
                                                                ;# TCP on/off: here we schedule the departure of the first flow
                                                                ;#             for each connection
                incr num_tcp
        }
}

if {$opt(tcp) == 1 && $opt(onoff) == 2} {
        puts $file(finish) "#\n#\tTCP Flows :\n#"

        HTTP_model_init

        set num_tcp 0

        foreach j [range 1 $opt(nn)] {
                set tcp($j) [new Agent/TCP/Newreno]     ;# Creates a new TCP NewReno agent          CLIENT
                set obj(tcp,$j) $tcp($j)
                $tcp($j) set fid_ $j                    ;# Sets the TCP flow identifier
                set sink($j) [new Agent/TCPSink]        ;# Creates a new TCP Sink agent
                set obj(tcp_sink,$j) $sink($j)
                $sink($j) set fid_ $j                   ;# Sets the TCP flow identifier

                set tcp([expr -$j]) [new Agent/TCP/Newreno]     ;# Creates a new TCP NewReno agent          SERVER
                set obj(tcp,[expr -$j]) $tcp([expr -$j])
                $tcp([expr -$j]) set fid_ [expr -$j]                    ;# Sets the TCP flow identifier
                set sink([expr -$j]) [new Agent/TCPSink]        ;# Creates a new TCP Sink agent
                set obj(tcp_sink,[expr -$j]) $sink([expr -$j])
                $sink($j) set fid_ [expr -$j]                   ;# Sets the TCP flow identifier


                puts $file(finish) [format "#\t\t%2d)  WN[expr $j - 1]-->0  :  client starting at %f" $j [expr 0.000050 * $num_tcp]]

                $ns_ attach-agent $node_($num_tcp) $tcp($j)
                $ns_ attach-agent $W(0) $sink($j)
                set fl($j) "WN[expr $j - 1]-->0"

                puts $file(finish) [format "#\t\t%2d)  0-->WN[expr $j - 1]  :  server waiting" $j]

                $ns_ attach-agent $W(0) $tcp([expr -$j])
                $ns_ attach-agent $node_($num_tcp) $sink([expr -$j])
                set fl([expr -$j]) "0-->WN[expr $j - 1]"


                $ns_ connect $tcp($j) $sink($j)         ;# Creates the transport level connection between the source and the destination
                $ns_ connect $tcp([expr -$j]) $sink([expr -$j])

                set ftp($j) [new Application/FTP]       ;# Creates a new FTP agent
                $ftp($j) attach-agent $tcp($j)          ;# Connects the FTP agent over the corresponding TCP one
                set ftp([expr -$j]) [new Application/FTP]
                $ftp([expr -$j]) attach-agent $tcp([expr -$j])

                set web($j,start) $opt(start)
                set web([expr -$j],start) $opt(start)

                $ns_ at [expr $opt(start) + [expr 0.000050 * $num_tcp]] "start_CS_flows [expr -$j]"

                incr num_tcp
        }
}

if {$opt(udp) == 1} {
        puts $file(finish) "#\n#\tCBR Flows :\n#"

        set num_udp 0

        foreach j [range 1 $opt(nn)] {
                set udp($j) [new Agent/UDP]     ;# Creates a new UDP agent
                set obj(udp,$j) $udp($j)
                $udp($j) set fid_ $j            ;# Sets the UDP flow identifier
                set null($j) [new Agent/Null]   ;# Creates a new UDP Sink agent (Null Agent)
                set obj(null,$j) $null($j)
                $null($j) set fid_ $j           ;# Sets the UDP flow identifier

                if {[expr $j % 4] == 2 || [expr $j % 4] == 0} {
                        puts $file(finish) [format "#\t\t%2d)  0-->WN[expr $j - 1]  :  down-link starting at %f" \
                                                        $j [expr 0.000050 * $num_udp]]
                        $ns_ attach-agent $W(0) $udp($j)
                        $ns_ attach-agent $node_($num_udp) $null($j)    ;#
                        set udp_fl($j) "0-->WN[expr $j - 1]"
                        set cbr($j) [new Application/Traffic/CBR]       ;# Creates a new CBR agent
                        $cbr($j) set rate_ 300000                       ;# Transmission rate (bit/sec)
                } else {
                        puts $file(finish) [format "#\t\t%2d)  WN[expr $j - 1]-->0  :  down-link starting at %f" \
                                                        $j [expr 0.000050 * $num_udp]]
                        $ns_ attach-agent $W(0) $udp($j)
                        $ns_ attach-agent $node_($num_udp) $null($j)               ;#
                        set udp_fl($j) "0-->WN[expr $j - 1]"
                        set cbr($j) [new Application/Traffic/CBR]       ;# Creates a new CBR agent
                        $cbr($j) set rate_ 64000                        ;# Transmission rate (bit/sec)
                }

                $ns_ connect $udp($j) $null($j)     ;# Creates the transport level connection between the source and the destination

                $cbr($j) attach-agent $udp($j)                  ;# Connects the CBR agent over the corresponding UDP one

                $ns_ at [expr $opt(start) + [expr 0.000050 * $num_udp]] "$cbr($j) start"
                $ns_ at $opt(stop) "$cbr($j) stop"
                                                        ;# Departure times of CBR sources are diversified among
                                                        ;#   stations to avoid a common collision at the beginning
                incr num_udp
        }
}

if {$opt(voip) == 1} {

        VoiceTracefile set debug_ 0
        VoiceTracefile set paretomean_ 4
        VoiceTracefile set paretoshape_ 2.5
        Application/Traffic/Trace/Voice set debug_ 0

        puts $file(finish) "#\n#\tVOIP Flows :\n#"

        set num_voip 0

        foreach j [range 1 $opt(nn)] {
                set voipc($j) [new Agent/UDP]     ;# Creates a new UDP agent
                set obj(voipc,$j) $voipc($j)
                $voipc($j) set fid_ $j            ;# Sets the UDP flow identifier
                set voips($j) [new Agent/UDP]
                set obj(voips,$j) $voips($j)
                $voips($j) set fid_ $j           ;# Sets the UDP flow identifier

                if {[expr $j % 4] == 2 || [expr $j % 4] == 0} {
                        puts $file(finish) [format "#\t\t%2d)  0-->WN[expr $j - 1]  :  down-link starting at %f" \
                                                        $j [expr 0.000050 * $num_voip]]
                        $ns_ attach-agent $W(0) $voipc($j)
                        $ns_ attach-agent $node_($num_voip) $voips($j)    ;#
                        set voip_fl($j) "0-->WN[expr $j - 1]"
                } else {
                        puts $file(finish) [format "#\t\t%2d)  WN[expr $j - 1]-->0  :  up-link starting at %f" \
                                                        $j [expr 0.000050 * $num_voip]]
                        $ns_ attach-agent $node_($num_voip) $voipc($j)
                        $ns_ attach-agent $W(0) $voips($j)               ;#
                        set voip_fl($j) "WN[expr $j - 1]-->0"
                }

                $ns_ connect $voipc($j) $voips($j)     ;# Creates the transport level connection between the source and the destination

                set tfile($j) [new VoiceTracefile]
		if {$opt(vocoder) == "g729"} {
                	$tfile($j) filename "g.729-vad.log"
		}
		if {$opt(vocoder) == "g711"} {
			$tfile($j) filename "g.711alaw-vad.log"
		}
                set t1($j) [new Application/Traffic/Trace/Voice]
                $t1($j) id 1
                $t1($j) attach-tracefile $tfile($j)
                $t1($j) attach-agent $voipc($j)

                set t2($j) [new Application/Traffic/Trace/Voice]
                $t2($j) id 2
                $t2($j) attach-tracefile $tfile($j)
                $t2($j) attach-agent $voips($j)

                $ns_ at [expr $opt(start) + [expr 0.000050 * $num_voip]] "$t1($j) start"
                $ns_ at [expr $opt(start) + [expr 0.000050 * $num_voip]] "$t2($j) start"
                $ns_ at $opt(stop) "$t1($j) stop"
                $ns_ at $opt(stop) "$t2($j) stop"

                incr num_voip
        }
}

set rng [new RNG]       ;# Creates a new random number generator
$rng seed $opt(seed)    ;# Sets the seed of the random number generator

######################################################################
#                                                                    #
# start_HTTP_flows: when the TCP on/off model is used, this          #
#                   function starts a new flow over the connection   #
#                                                                    #
#          fid: the flow identifier of this TCP connection           #
#                                                                    #
######################################################################

proc start_HTTP_flows {fid} {

	global rng web ns_ avg opt ftp tcp obj stat
	set DELAY($fid) [expr $opt(avg_wait) * [$rng exponential]]        ;# The silence period between two consecutive flows is
	set next [expr [$ns_ now] + $DELAY($fid)]                         ;# chosen according to an exponential distribution with
        set B2S $avg(flow_length,[$rng integer $web(npkt)])               ;# average value equal to 'opt(avg_wait)'
        set P2S [expr 1 + int($B2S/$opt(tcp_size))]
        set web($fid,start) $next                                         ;# The number of packet to send is uniformly extracted
        set web($fid,p2s) $P2S                                            ;# among the ones defined in HTTP_model_init
	if {$next <= $opt(stop)} {
                set stat(tcp,idle_time,$fid) [expr $stat(tcp,idle_time,$fid) + $DELAY($fid)]
                incr stat(tcp,n_flow,$fid)
                $ns_ at $next "$obj(tcp,$fid) reset"              ;# Before transmit again, we reset the TCP variables
                $ns_ at $next "$obj(tcp_sink,$fid) reset"
		$ns_ at $next "$ftp($fid) producemore $P2S"       ;# Gives to FTP the command to produce new packets
	} elseif {[$ns_ now] < $opt(stop)} {
                set stat(tcp,idle_time,$fid) [expr $stat(tcp,idle_time,$fid) + $opt(stop) - [$ns_ now]]
        }
}

proc start_CS_flows {fid} {
	global rng web ns_ avg opt ftp tcp obj stat

        if {$fid < 0} {
	        set B2S $avg(flow_length,[$rng integer 10])
        } else {
	        set B2S $avg(flow_length,[$rng integer 15])
        }
        set P2S [expr 1 + int($B2S/$opt(tcp_size))]

        set web([expr -$fid],start) [$ns_ now]
        set web([expr -$fid],p2s) $P2S

	if {[$ns_ now] <= $opt(stop)} {
                set stat(tcp,idle_time,[expr -$fid]) [expr $stat(tcp,idle_time,[expr -$fid]) + [$ns_ now] - $web($fid,start)]
                incr stat(tcp,n_flow,[expr -$fid])
                $ns_ at [$ns_ now] "$obj(tcp,[expr -$fid]) reset"
                $ns_ at [$ns_ now] "$obj(tcp_sink,[expr -$fid]) reset"
		$ns_ at [$ns_ now] "$ftp([expr -$fid]) producemore $P2S"
	} else {
                set stat(tcp,idle_time,[expr -$fid]) [expr $stat(tcp,idle_time,[expr -$fid]) + $opt(stop) - $web($fid,start)]
        }
}






